package com.malk.rongzhi;

import com.malk.Util.UtilHttp;
import com.malk.Util.UtilMap;
import com.malk.server.common.McException;
import com.malk.server.common.McR;
import com.malk.server.common.VenR;
import com.malk.server.dingtalk.DDConf;
import com.malk.service.dingtalk.DDClient_Contacts;
import lombok.Data;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 错误抛出与拦截详见CatchException
 */
@Slf4j
@RestController
@RequestMapping("/rongzhi")
public class RongZhiController {

    // 请求类型枚举, 避用条件判定
    private enum TYPE {
        dingtalk("/dingding"),
        ekuaibao("/ekuaibao");

        @Getter
        private String path;

        TYPE(String path) {
            this.path = path;
        }
    }

    /// 避免内部类和嵌套类, 不能做序列化, 内部类添加static关键字
    @Data
    private static class RZR extends VenR {
        private String err_code;
        private String message;
        private String data;

        // 成功状态标记
        private final static String SUC_CODE = "0";

        /**
         * 断言错误信息
         */
        @Override
        public void assertSuccess() {
            McException.assertException(!err_code.equals(SUC_CODE), err_code, message, "MC-RZ");
        }
    }

    private static String nonce = "39350e21-d28c-423d-ae1c-97d9fdac6a60";

    private String getAccessToken(TYPE type) {
        String timestamp = String.valueOf(new Date().getTime());
        String signature = DigestUtils.md5Hex("Max" + timestamp + nonce);
        Map header = new HashMap();
        header.put("timestamp", timestamp);
        header.put("nonce", nonce);
        header.put("signature", signature);
        RZR rsp = (RZR) UtilHttp.doGet("https://one-api.max-digital.cn/v1/token" + type.path, header, null, RZR.class);
        return rsp.data;
    }


    @Autowired
    private DDClient_Contacts ddClientImpl_contacts;

    /**
     * 阿里发票验真
     */
    @PostMapping("/invoice/check")
    McR invoiceCheckPDF(@RequestBody Map<String, String> param) {

        ddClientImpl_contacts.listSubDepartmentId(UtilMap.map("access_token", getAccessToken(TYPE.dingtalk)), UtilMap.map("dept_id", DDConf.TOP_DEPARTMENT)).forEach(deptId -> {
            

            List<String> userIds = ddClientImpl_contacts.listDepartmentUserId(UtilMap.map("access_token", getAccessToken(TYPE.dingtalk)), UtilMap.map("dept_id", DDConf.TOP_DEPARTMENT));
            log.info("xxxx, {}, {}", userIds.size(), userIds);
        });


        return McR.success(UtilMap.map("q, b, c", getAccessToken(TYPE.dingtalk), "xx", "xxxxx"));
    }

}
