package com.malk.xinwei.controller;

import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilServlet;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.server.common.McR;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliyun.ALYInvoice;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 错误抛出与拦截详见 CatchException
 */
@Slf4j
@RestController
@RequestMapping("/chuoqi")
@EnableAsync
public class ShiShangController {

    @Autowired
    private ALYInvoice invoice;

    @Autowired
    private YDClient ydClient;

    /**** 拾上服务 ****/

    /**
     * 阿里发票验真
     */
    @PostMapping("invoice/check")
    McR invoiceCheckPDF(@RequestBody Map<String, String> param) {
        log.info("阿里发票验真, 文件, {}", param);
        return McR.success(invoice.invoiceCheckPDF("3011b2ee820744679081f314d983b6bd", param.get("url")));
    }

    /**
     * 阿里发票验真: 参数
     */
    @PostMapping("invoice/verify")
    McR invoiceCheckParam(@RequestBody Map<String, String> param) {
        log.info("阿里发票验真, 参数, {}", param);
        String kprq = param.get("kprq").replace("年", "").replace("月", "").replace("日", "").replace("-", "").replace("/", "");
        return McR.success(invoice.invoiceCheckCode("3011b2ee820744679081f314d983b6bd", param.get("fpdm"), param.get("fphm"), kprq, param.get("noTaxAmount"), param.get("checkCode")));
    }

    /**
     * 阿里发票验真: image
     */
    @PostMapping("invoice/ocr")
    McR invoiceCheckImage(@RequestBody Map<String, String> param) {
        log.info("阿里发票验真, 图片, {}", param);
        return McR.success(invoice.invoiceCheckOCR("3011b2ee820744679081f314d983b6bd", param.get("url")));
    }

    /**
     * 退回重新提交明细处理
     */
    @PostMapping("invoice/return")
    McR invoiceSendBack(@RequestBody Map param) {
        log.info("退回重新提交明细处理, {}", param);
        McException.assertParamException_Null(UtilServlet.isNull(param, "pre_ids", "pre_update", "cur_ids", "cur_update"));
        YDParam ydParam = YDParam.builder()
                .appType("APP_NRRPLFQ8GEEURWIFNYB0")
                .systemToken("XJ866N71SJZ1OZ7WBUIL7C9SXLBO2QO4JDG5LDO")
                .build();
        List<String> pre_ids = (List<String>) param.get("pre_ids");
        List<String> cur_ids = (List<String>) param.get("cur_ids");
        // 释放状态实例Id
        List free_ids = pre_ids.stream().filter(t -> !cur_ids.contains(t)).collect(Collectors.toList());
        log.info("退回实例 {}, 占用实例, {}", free_ids, cur_ids);
        free_ids.forEach(id -> {
            ydParam.setFormInstId(String.valueOf(id));
            ydParam.setUpdateFormDataJson(JSON.toJSONString(param.get("pre_update")));
            ydClient.operateData(YDConf.API_FORM_UPDATE, ydParam);
        });
        // 占用选中实例Id
        cur_ids.forEach(id -> {
            ydParam.setFormInstId(String.valueOf(id));
            ydParam.setUpdateFormDataJson(JSON.toJSONString(param.get("cur_update")));
            ydClient.operateData(YDConf.API_FORM_UPDATE, ydParam);
        });
        return McR.success();
    }

    /**
     * 宜搭临时免登地址: 宜搭版本
     */
    @PostMapping("yd/openUrl")
    McR openUrl_YD(@RequestBody Map<String, String> param) {
        McException.assertParamException_Null(param, "url");
        YDParam ydParam = YDParam.builder()
                .appType("APP_NRRPLFQ8GEEURWIFNYB0")
                .systemToken("XJ866N71SJZ1OZ7WBUIL7C9SXLBO2QO4JDG5LDO")
                .fileUrl(param.get("url"))
                .build();
        return McR.success(ydClient.queryData(YDConf.API_OPEN_URL, ydParam));
    }

    @PostMapping("/portal")
    List<Map> portal() {

        List<Map> list = new ArrayList<>();
        Map item = new HashMap();
        item.put("image", "https://gw.alicdn.com/tfs/TB1fcfbsY9YBuNjy0FgXXcxcXXa-1200-675.jpg");
        item.put("title", "Arralis要做下一代毫米波雷达");
        item.put("dateTime", "2018-11-01 12:45");
        item.put("source", "'36Kr'");
        item.put("source", "'https://www.baidu.com'");
        list.add(item);

        return list;
    }
}
