package com.malk.Util;

import com.malk.server.common.McException;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;

public class UtilMap {

    // 工具方法
    public static Consumer<Object> consumerWithIndex(BiConsumer<Object, Integer> consumer) {
        class Obj {
            int i;
        }
        Obj obj = new Obj();
        return t -> {
            int index = obj.i++;
            consumer.accept(t, index);
        };
    }

    // 快速创建map
    public static Map<String, Object> map(String keys, Object... values) {
        String[] props = keys.split(", ");
        if (props.length != values.length) {
            McException.assertParamException_Null(keys);
        }
        Map<String, Object> map = new HashMap<>();
        Arrays.stream(values).forEach(consumerWithIndex((item, index) -> {
            map.put(props[index], item);
        }));
        return map;
    }

    // 快速创建map
    public static Map<String, String> map(String keys, String values) {
        String[] props = keys.split(", ");
        String[] contents = values.split(", ");
        if (props.length != contents.length) {
            McException.assertParamException_Null(keys);
        }
        Map<String, String> map = new HashMap<>();
        Arrays.stream(contents).forEach(consumerWithIndex((item, index) -> {
            map.put(props[index], String.valueOf(item));
        }));
        return map;
    }

    // 快速创建map
    public static Map<String, Object> map(String skeys, String ckeys, Map map) {
        String[] sprops = skeys.split(", ");
        String[] cprops = ckeys.split(", ");
        if (sprops.length != cprops.length) {
            McException.assertParamException_Null(skeys);
        }
        Map data = new HashMap();
        Arrays.stream(cprops).forEach(consumerWithIndex((item, index) -> {
            data.put(sprops[index], map.get(item));
        }));
        return data;
    }

    // 快速创建map
    public static Map<String, Object> map(String skeys, String ckeys, Map map, boolean isReserve) {
        if (isReserve) {
            map.putAll(map(skeys, ckeys, map));
            return map;
        }
        return map(skeys, ckeys, map);
    }

    // distinctByKey: 从人员列表中过滤出一个子集(每个部门选一个人) employees.stream().filter(distinctByKey(Employee::getDeptCode)).collect(toList());
    public static <T> Predicate<T> distinctByKey(Function<? super T, ?> keyExtractor) {
        Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        return t -> Objects.isNull(seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE));
    }

}
