package com.malk.controller;

import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McR;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 错误抛出与拦截详见CatchException
 */

@Api(tags = "宜搭", value = "YD", description = "宜搭相关接口")
@RestController
@RequestMapping("/aliwork")
public class McAliworkController {

    @Autowired
    private YDService service;

    @Autowired
    private YDClient client;

    ////////////////////////////////////////
    // 原子接口
    ////////////////////////////////////////
    
    @ApiOperation(tags = {"mc"}, value = "表单数据查询", notes = "匹配条件, 查询表单, 返回formData列表")
    @PostMapping("form/queryDataList")
    McR queryFormDataMatchCondition(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(client.queryData(YDConf.API_FORM_QUERY_DATA, param));
    }

    /**
     * 2.匹配条件, 查询流程, 返回data列表
     */
    @ApiOperation(value = "登1录", notes = "用户登录", httpMethod = "POST")
    @PostMapping("process/queryDataList")
    McR queryProcessDataMatchCondition(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(client.queryData(YDConf.API_PROCESS_QUERY_DATA, param));
    }

    /**
     * 3.匹配条件, 查询表单, 返回formInstId列表
     */
    @PostMapping("form/queryIdList")
    McR queryFormInstIdMatchCondition(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(client.queryData(YDConf.API_FORM_QUERY_ID, param));
    }

    /**
     * 4.匹配条件, 查询流程, 返回processInstanceId列表
     */
    @PostMapping("process/queryIdList")
    McR queryProcessInstanceIdMatchCondition(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(client.queryData(YDConf.API_FORM_QUERY_ID, param));
    }

    /**
     * 5.匹配formInstId, 返回表单实例详情
     */
    @PostMapping("form/queryDetail")
    McR queryFormDetail(@RequestBody @Validated(YDParam.Retrieve_FormInstId.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_FORM_DETAIL, param));
    }

    /**
     * 6.匹配processInstanceId, 返回流程实例详情
     */
    @PostMapping("process/queryDetail")
    McR queryProcessDetail(@RequestBody @Validated(YDParam.Retrieve_ProcessInstanceId.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_PROCESS_DETAIL, param));
    }

    /**
     * 7.匹配processInstanceIds, 批量返回流程实例详情列表
     */
    @PostMapping("process/queryBatchDetail")
    McR queryProcessBatchDetail(@RequestBody @Validated(YDParam.Retrieve_ProcessInstanceIds.class) YDParam param) {
        return McR.success(client.queryData(YDConf.API_PROCESS_BATCH_DETAIL, param));
    }

    /**
     * 8.新增表单数据, result返回实例Id
     */
    @PostMapping("form/create")
    McR createFormData(@RequestBody @Validated(YDParam.Create.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_FORM_CREATE, param));
    }

    /**
     * 9.新增流程数据, result返回实例Id
     */
    @PostMapping("proces/create")
    McR createProcessData(@RequestBody @Validated(YDParam.Create_Process.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_PROCESS_CREATE, param));
    }

    /**
     * 10.删除表单, 返回result为空, 若失败抛出异常于过滤器处理
     */
    @PostMapping("form/delete")
    McR deleteFormData(@RequestBody @Validated(YDParam.Delete.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_FORM_DELETE, param));
    }

    /**
     * 11.删除流程, 返回result为空, 若失败抛出异常于过滤器处理
     */
    @PostMapping("process/delete")
    McR deleteProcessData(@RequestBody @Validated(YDParam.Delete_ProcessInstanceId.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_PROCESS_DELETE, param));
    }

    /**
     * 12.更新表单, 返回result为空, 若失败抛出异常于过滤器处理
     */
    @PostMapping("form/upate")
    McR updateFormData(@RequestBody @Validated(YDParam.Update.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_FORM_UPDATE, param));
    }

    /**
     * 13.更新表单, 返回result为空, 若失败抛出异常于过滤器处理
     */
    @PostMapping("process/upate")
    McR updateProcessData(@RequestBody @Validated(YDParam.Update_ProcessInstanceId.class) YDParam param) {
        return McR.success(client.operateData(YDConf.API_FORM_UPDATE, param));
    }


    ////////////////////////////////////////
    // 服务接口
    ////////////////////////////////////////

    /**
     * a. 并发查询全量数据
     */
    @PostMapping("server/queryAllByConcurrence")
    McR queryAllDataByConcurrence(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(service.queryAllDataByConcurrence(param));
    }

    /**
     * b. 同步查询全量数据
     */
    @PostMapping("server/queryAll")
    McR queryAllData(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(service.queryAllData(param));
    }

    /**
     * c. 查询条件实现: 批量删除, 一个分页
     */
    @PostMapping("server/batchDelete")
    McR batchDelete(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(service.operatePageData(param));
    }

    /**
     * d. 查询条件实现: 批量更新, 一个分页
     */
    @PostMapping("server/batchUpdate")
    McR batchUpdate(@RequestBody @Validated({YDParam.Retrieve_Condition_Update.class}) YDParam param) {
        return McR.success(service.operatePageData(param));
    }

    /**
     * e. 查询条件实现: 批量删除, 一个切片
     */
    @PostMapping("server/limitDelete")
    McR limitDelete(@RequestBody @Validated({YDParam.Retrieve_Condition.class}) YDParam param) {
        return McR.success(service.operateLimitData(param));
    }

    /**
     * f. 查询条件实现: 批量更新, 一个切片
     */
    @PostMapping("server/limitUpdate")
    McR limitUpdate(@RequestBody @Validated(YDParam.Retrieve_Condition_Update.class) YDParam param) {
        return McR.success(service.operateLimitData(param));
    }

    /**
     * g. 查询条件实现: 全量删除
     */
    @PostMapping("server/deleteAll")
    McR deleteAll(@RequestBody @Validated({YDParam.Retrieve_Condition.class}) YDParam param) {
        return McR.success(service.operateAllData(param));
    }

    /**
     * h. 查询条件实现: 全量更新
     */
    @PostMapping("server/updateALl")
    McR updateAll(@RequestBody @Validated(YDParam.Retrieve_Condition_Update.class) YDParam param) {
        return McR.success(service.operateAllData(param));
    }
}
