package com.malk.controller;

import cn.hutool.core.util.ObjectUtil;
import com.malk.repository.dao.mutual.McAuthorizationDao;
import com.malk.repository.entity.mutual.McAuthorizationDTO;
import com.malk.server.common.McR;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 错误抛出与拦截详见 CatchException
 * -
 * 关于入参为JSONString类型: body内的json需要传入字符串, 若为对象会解析报错. 若是formData, 传入对象会自动转为字符串, 不会解析错误, 无需转为JSONString
 * 关于Body（json）和FormData的说明：首先若是接收为实体类，添加@RequestBody可实例化JSON格式，不加则实例化是FormData格式。无实体前者是Map，后者需要通过request转map
 */
@Api(tags = "mc", value = "mcli", description = "mjava 相关接口")
@Slf4j
@RestController
@RequestMapping("/licence")
public class McLicenceController {

    @Autowired
    McAuthorizationDao useAuthDao;

    @ApiOperation(tags = {"mc", "宜搭"}, value = "申请宜搭接口授权", notes = "申请通过后, 才可使用 mjava 相关接口能力")
    @PostMapping("auth/approve")
    McR approve(@RequestBody @Validated McAuthorizationDTO useAuthDo) {
        McAuthorizationDTO recordDo = useAuthDao.findByAppType(useAuthDo.getAppType());
        // jda之save接口会会以传入数据为准，[忽略空字段避免被覆盖]
        if (ObjectUtil.isNotNull(recordDo)) {
            recordDo.mergeParam(useAuthDo);
        }
        useAuthDao.save(useAuthDo);
        return McR.success();
    }

    @ApiOperation(tags = {"mc", "宜搭"}, value = "查询宜搭接口授权", notes = "查询 appType, 应用是否取得调用权限")
    @PostMapping("auth/detail")
    McR detail(@RequestParam String appType) {
        log.info("查询授权, {}", appType);
        return McR.success(useAuthDao.findByAppType(appType));
    }
}
