package com.malk.controller;

import com.malk.Util.UtilServlet;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.common.McException;
import com.malk.server.common.McR;
import com.malk.server.teambition.TBConf;
import com.malk.service.aliwork.YDClient_DD;
import com.malk.service.teambition.TBClient;
import com.malk.service.teambition.TBService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * TB-YD: tb2yd对接方案
 */
@Slf4j
@RestController
@RequestMapping("/tb")
public class TB2YDController {

    @Autowired
    private TBClient tbClient;

    @Autowired
    private TBConf tbConf;

    @Autowired
    private YDConf ydConf;

    @Autowired
    private YDClient_DD ydClient_dd;

    @Autowired
    private TBService tbService;

    /**
     * 搜索企业模板模板
     */
    @PostMapping("template")
    public McR getTemplateList(@RequestBody Map<String, String> data) {
        return McR.success(tbClient.templateSearch(data.get("id"), data.get("name")));
    }

    /**
     * 搜索用户
     */
    @GetMapping("user")
    public McR searchUser(@RequestParam String search) {
        return McR.success(tbClient.userSearch(search));
    }

    /**
     * 更新用户信息
     */
    @PutMapping("user")
    public McR updateUser(@RequestBody Map<String, ?> data) {
        McException.assertParamException_Null(UtilServlet.isNull(data, "members"));
        String operatorId = "";
        if (UtilServlet.isNotNull(data, "userName")) {
            operatorId = tbService.convertUserIdWithSearch(String.valueOf(data.get("userName"))).get(0);
        }
        return McR.success(tbClient.updateUser(operatorId, (List<Map>) data.get("members")));
    }

    /**
     * 创建空白项目, 通过模板创建项目
     */
    @SneakyThrows
    @PostMapping("project")
    public McR createProjectWithTemplate(@RequestBody Map<String, String> data) {
        McException.assertParamException_Null(UtilServlet.isNull(data, "projectName", "userName"));
        String operatorId = tbService.convertUserIdWithSearch(String.valueOf(data.get("userName"))).get(0);
        Map projectInfo = null;
        if (UtilServlet.isNotNull(data, "templateId")) {
            projectInfo = tbClient.projectCreateWithTemplate(data.get("projectName"), data.get("templateId"), operatorId);
        } else {
            projectInfo = tbClient.projectCreate(data.get("projectName"), operatorId);
        }
        // 更新宜搭 & tb项目信息
        tbService.updateProject_YD_TB(projectInfo, data.get("userName"));
        return McR.success(projectInfo);
    }

    /**
     * 更新项目
     */
    @PutMapping("project/{id}")
    public McR updateProject(@PathVariable String id, @RequestBody Map<String, String> data) {
        McException.assertParamException_Null(UtilServlet.isNull(data, "userName"));
        String operatorId = tbService.convertUserIdWithSearch(String.valueOf(data.get("userName"))).get(0);
        return McR.success(tbClient.projectUpdate(id, data, operatorId));
    }

    /**
     * 创建项目成员, 修改项目成员的角色
     */
    @RequestMapping(value = "project/member", method = {RequestMethod.POST, RequestMethod.PUT})
    public McR createProjectMember(@RequestBody Map<String, Object> data) {
        McException.assertParamException_Null(UtilServlet.isNull(data, "userName"));
        List userIds = tbService.convertUserIdWithSearch_bulk(data);
        if (UtilServlet.isNotNull(data, "roleIds")) {
            return McR.success(tbClient.updateProjectMember(userIds, (List<String>) data.get("roleIds"), String.valueOf(data.get("projectId"))));
        }
        McException.assertParamException_Null(UtilServlet.isNull(data, "userName"));
        String operatorId = tbService.convertUserIdWithSearch(String.valueOf(data.get("userName"))).get(0);
        return McR.success(tbClient.createProjectMember(userIds, String.valueOf(data.get("projectId")), operatorId));
    }

    /**
     * 获取项目角色列表
     */
    @GetMapping("{projectId}/role")
    McR queryProjectRoles(@PathVariable String projectId) {
        return McR.success(tbClient.queryProjectRoles(projectId));
    }

    /**
     * 更新任务截止时间
     */
    @PutMapping("task/{id}")
    McR updateTaskDueDate(@PathVariable String id, @RequestBody Map<String, String> data) {
        McException.assertParamException_Null(UtilServlet.isNull(data, "userName", "dueDate"));
        String operatorId = tbService.convertUserIdWithSearch(String.valueOf(data.get("userName"))).get(0);
        return McR.success(tbClient.updateTeakDueDate(id, data.get("dueDate"), operatorId));
    }

    /**
     * 新增表单数据
     */
    @PostMapping("form")
    McR createFormData(@RequestBody @Validated(YDParam.Create.class) YDParam param) {
        return McR.success(ydClient_dd.operateData(param, YDConf.TYPE_OPERATION.create));
    }

    /**
     * 更新表单数据
     */
    @PutMapping("form")
    McR updateFormData(@RequestBody @Validated(YDParam.Update.class) YDParam param) {
        return McR.success(ydClient_dd.operateData(param, YDConf.TYPE_OPERATION.update));
    }

    /**
     * 表单数据列表
     */
    @GetMapping("form")
    McR queryFormData(@RequestBody @Validated(YDParam.Retrieve_Condition.class) YDParam param) {
        return McR.success(ydClient_dd.queryData(param, YDConf.TYPE_QUERY.retrieve_list));
    }
}
