package com.malk.server.aliwork;

import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

@Data
@Component
@ConfigurationProperties(prefix = "aliwork")
public class YDConf {

    private String appType;

    private String systemToken;

    /**
     * 一个切片数量上限
     */
    public static final Integer UPPER_LIMIT = 30000;

    /**
     * 一个分页数量上限
     */
    public static final Integer PAGE_SIZE_LIMIT = 100;

    /**
     * 接口访问账号
     * -
     * 非管理员账号只可以查询, 不检验权限
     * 操作数据需要管理员账号, 或宜搭平台
     */
    public static final String PUB_ACCOUNT = "yida_pub_account";

    ////////////////////////// 新版本API //////////////////////////

    /**
     * 查询表单
     */
    public enum TYPE_QUERY {
        retrieve_id,                // 单个ID查询
        retrieve_search,            // 条件查询, 返回数据列表
        retrieve_search_id,         // 条件查询, 返回实例ID列表
        retrieve_list,              // 全局查询, 不包含子表单
        retrieve_list_all,          // 全局查询, 包含子表单
        multi_retrieve_id,
    }

    /**
     * 数据操作
     */
    public enum TYPE_OPERATION {
        create,
        delete,
        update,
        multi_create,               // 批量操作
        multi_delete,
        multi_update,
    }

    ////////////////////////// 老版本API //////////////////////////

    /**
     * 表单接口适用于流程: 查询和更新以及删除
     */
    private static String formatApiForm(String uri) {
        return "/yida_vpc/form/" + uri + ".json";
    }

    private static String formatApiProcess(String uri) {
        return "/yida_vpc/process/" + uri + ".json";
    }

    /**
     * 接口地址: 表单
     */
    public static final String API_FORM_CREATE = formatApiForm("saveFormData");
    public static final String API_FORM_DELETE = formatApiForm("deleteFormData");
    public static final String API_FORM_UPDATE = formatApiForm("updateFormData");
    public static final String API_FORM_DETAIL = formatApiForm("getFormDataById");
    public static final String API_FORM_QUERY_ID = formatApiForm("searchFormDataIds");
    public static final String API_FORM_QUERY_DATA = formatApiForm("searchFormDatas");

    public static final String API_FORM_DEFINITION = "/yida_vpc/formDesign/getFormComponentDefinationList.json";

    /**
     * 接口地址: 流程
     */
    public static final String API_PROCESS_CREATE = formatApiProcess("startInstance");
    public static final String API_PROCESS_DELETE = formatApiProcess("deleteInstance");
    public static final String API_PROCESS_UPDATE = formatApiProcess("updateInstance");
    public static final String API_PROCESS_DETAIL = formatApiProcess("getInstanceById");
    public static final String API_PROCESS_BATCH_DETAIL = formatApiProcess("getInstancesByIds");
    public static final String API_PROCESS_QUERY_ID = formatApiProcess("getInstanceIds");
    public static final String API_PROCESS_QUERY_DATA = formatApiProcess("getInstances");

    /**
     * 其它接口
     */
    public static final String API_OPEN_URL = "/yida_vpc/file/getOpenUrl.json";
}
