package com.malk.server.aliwork;

import com.alibaba.fastjson.JSONObject;
import com.malk.base.BaseParam;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.commons.lang3.StringUtils;

import javax.validation.constraints.NotNull;
import javax.validation.constraints.Null;
import javax.validation.groups.Default;

/**
 * 表单接口适用于流程: 查询和更新以及删除 [参数校验参考CatchException]
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
public class YDParam extends BaseParam {

    /**
     * 接口参数
     */
    @NotNull(message = "应用编码不能为空")
    private String appType;

    @Null(message = "应用秘钥无需传递")
    private String systemToken;

    // 默认宜搭平台, 有操作数据权限
    @Builder.Default
    private String userId = YDConf.PUB_ACCOUNT;

    @NotNull(message = "表单编码不能为空", groups = {Create.class, Retrieve_Condition.class, Definition.class, Create_Process.class, Retrieve_Condition_Update.class})
    private String formUuid;

    private String searchFieldJson;

    private Integer currentPage;

    private Integer pageSize;

    // 流程暂不支持排序
    private String dynamicOrder;

    @NotNull(message = "实例ID不能为空", groups = {Update.class, Delete.class, Retrieve_FormInstId.class})
    private String formInstId;

    @NotNull(message = "更新内容不能为空", groups = {Update.class, Update_ProcessInstanceId.class, Retrieve_Condition_Update.class})
    private String updateFormDataJson;

    @NotNull(message = "流程编码不能为空", groups = Create_Process.class)
    private String processCode;

    @NotNull(message = "新增内容不能为空", groups = {Create.class, Create_Process.class})
    public String formDataJson;

    @NotNull(message = "实例ID不能为空", groups = {Update_ProcessInstanceId.class, Delete_ProcessInstanceId.class, Retrieve_ProcessInstanceId.class})
    private String processInstanceId;

    @NotNull(message = "实例ID不能为空", groups = Retrieve_ProcessInstanceIds.class)
    private String processInstanceIds;

    private String instanceStatus;

    private String approvedResult;

    /**
     * 格式化赋值: 内部使用
     */
    public void setDynamicOrder(String dynamicOrder) {
        this.dynamicOrder = dynamicOrder;
    }

    public void setDynamicOrder(String rule, String... compIds) {
        for (String compId : compIds) {
            JSONObject orderJson = new JSONObject();
            orderJson.put(compId, rule);
            this.dynamicOrder = orderJson.toJSONString();
        }
    }

    /**
     * 自定义参数
     */
    private String compIdSerial; // 流水号排序字段

    /**
     * 钉钉新版本接口
     */
    private String formInstanceId;

    // formInstanceId 为新版本实例id字段
    public String getFormInstanceId() {
        if (StringUtils.isBlank(formInstanceId)) {
            return formInstId;
        }
        return formInstanceId;
    }

    private String searchCondition;

    // 全局条件查询列表数据
    public String getSearchCondition() {
        if (StringUtils.isBlank(searchCondition)) {
            return searchFieldJson;
        }
        return searchCondition;
    }

    // 查询列表可查询多个指定实例ID
    private String[] formInstanceIdList;

    // 是否需要宜搭表单组件格式的实例数据
    private boolean needFormInstanceValue = false;

    // 附件转临时免登地址
    private String fileUrl;
    // 临时地址失效时间, 默认10分钟, 最大值24小时
    @Builder.Default
    private Integer timeout = 600000;

    /**
     * 分组校验
     *
     * @RequestBody @Validated(YDParam.Create.class) YDParam ydParam
     */

    public interface Create extends Default {

    }

    public interface Create_Process extends Default {

    }

    public interface Retrieve_Condition extends Default {

    }

    public interface Retrieve_Condition_Update extends Default {

    }

    public interface Retrieve_FormInstId extends Default {

    }

    public interface Retrieve_ProcessInstanceId extends Default {

    }

    public interface Retrieve_ProcessInstanceIds extends Default {

    }

    public interface Update extends Default {

    }

    public interface Update_ProcessInstanceId extends Default {

    }

    public interface Delete extends Default {

    }

    public interface Delete_ProcessInstanceId extends Default {

    }

    public interface Definition extends Default {

    }
}
