package com.malk.server.feishu;


import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

/**
 * 读取配置文件参考FilePah
 */
@Data
@Component
@ConfigurationProperties(prefix = "feishu")
public class FSConf {

    private String appKey;

    private String appSecret;

    private String encryptKey;

    private String tenantKey;

    // NOTE: 没有实际用途, 会在回调里面返回, 不参与加密, 官方文档提示可用于校验是否飞书回调 [钉钉token和key参与加密, 且token不返回]
    private String verToken;

    /**
     * 一个分页数量上限
     */
    public static final Integer PAGE_SIZE_LIMIT = 50;

    /**
     * 飞书一级部门: 0
     */
    public static final String TOP_DEPARTMENT = "0";

    /**
     * 验证注册地址, 通过开发平台配置触发
     */
    public final static String CALLBACK_CHECK = "url_verification";

    /**
     * 审批事件回调, 监听审批模板审批触发
     */
    public final static String CALLBACK_EVENT = "event_callback";


    /**
     * 用户打卡事件回调
     */
    public final static String EVENT_ATTENDANCE = "attendance.user_flow.created_v1";

    /**
     * 审批事件: 任务
     */
    public final static String EVENT_TASK = "approval_task";

    /**
     * 审批事件: 结果
     */
    public final static String EVENT_INSTANCE = "approval_instance";

    /**
     * 任务触发: 转交/撤回/退回, 状态为 DOME
     */
    public final static String STATUS_DONE = "DONE";

    /**
     * 任务/结果触发: 发起/退回/同意/拒绝/重新提交, 状态为 PENDING [结果触发: 没有任何审批就撤回/发起完成后/转交完成后]
     */
    public final static String STATUS_PENDING = "PENDING";

    /**
     * 任务触发: 转交
     */
    public final static String STATUS_TRANSFERRED = "TRANSFERRED";

    /**
     * 结果触发, 撤回 (审批中取消), 状态为 CANCELED
     */
    public final static String STATUS_CANCELED = "CANCELED";

    /**
     * 结果触发, 撤销 (通过申请撤销后重新审批通过), 状态为 REVERTED
     */
    public final static String STATUS_REVERTED = "REVERTED";

    /**
     * 任务/结果触发, 同意/退回, 状态都是 APPROVED
     */
    public final static String STATUS_APPROVED = "APPROVED";

    /**
     * 任务/结果触发: 拒绝, 状态都是 APPROVED
     */
    public final static String STATUS_REJECTED = "REJECTED";

    /**
     * 任务/结果触发: 删除, 审批事件类型也是 approval_instance, 状态都是 DELETED [已经完成的数据包含撤销, 不会进行回调]
     */
    public final static String STATUS_DELETED = "DELETED";

    /**
     * 审批状态转文本描述
     */
    public static String convertApproveStatus(String status) {
        Map<String, String> state = new HashMap<>();
        state.put("PENDING", "审批中");
        state.put("APPROVED", "已通过");
        state.put("REJECTED", "已拒绝");
        state.put("CANCELED", "已撤回");
        state.put("DELETED", "已删除");
        state.put("REVERTED", "已撤销");
        return Optional.ofNullable(state.get(status)).orElse(status);
    }
}
