package com.malk.server.feishu;

import com.alibaba.fastjson.JSONObject;
import com.malk.base.BaseParam;
import lombok.Data;

/**
 * 参数校验参考CatchException
 */
@Data
public class FSParam extends BaseParam {

    /**
     * 审批结果写入: 请假表体内容
     *
     * @param approval_id 审批实例ID
     * @param duration    加班时长
     * @param unit        加班时长单位，可用值：【1（天），2（小时）】
     * @param category    加班日期类型，可用值：【1（工作日），2（休息日），3（节假日）】
     * @param type        加班规则类型，可用值：【0（不关联加班规则），1（调休），2（加班费）】
     * @param start_time  开始时间，时间格式为 yyyy-MM-dd HH:mm:ss
     * @param end_time    结束时间，时间格式为 yyyy-MM-dd HH:mm:ss
     * @return overtime_works 表体
     */
    public static JSONObject overTimeContent(String approval_id, float duration, int unit, int category, int type, String start_time, String end_time) {
        JSONObject content = new JSONObject();
        content.put("approval_id", approval_id);
        content.put("duration", duration);
        content.put("unit", unit);
        content.put("category", category);
        content.put("type", type);
        content.put("start_time", start_time);
        content.put("end_time", end_time);
        return content;
    }

    /**
     * 审批结果写入: 请假表体内容
     *
     * @param uniq_id     假期类型唯一 ID，代表一种假期类型，长度小于 14
     * @param unit        假期时长单位，可用值：【1（天），2（小时），3（半天），4（半小时）】
     * @param interval    假期时长（单位秒）
     * @param start_time  开始时间，时间格式为 yyyy-MM-dd HH:mm:ss
     * @param end_time    结束时间，时间格式为 yyyy-MM-dd HH:mm:ss
     * @param reason      请假理由，必选字段
     * @param description 描述信息: 可适配多语言, 会显示于后台请假说明
     * @return leaves 表体
     */
    public static JSONObject leaveContent(String uniq_id, int unit, int interval, String start_time, String end_time, String reason, String description) {
        JSONObject content = new JSONObject();
        content.put("uniq_id", uniq_id);
        content.put("unit", unit);
        content.put("interval", interval);
        content.put("start_time", start_time);
        content.put("end_time", end_time);
        content.put("reason", reason);
        // 默认语言类型
        JSONObject i18n = new JSONObject();
        i18n.put("ch", description);
        i18n.put("en", description);
        i18n.put("ja", description);
        content.put("i18n_names", i18n);
        content.put("default_locale", "ch");
        return content;
    }
}
