package com.malk.service.aliwork.impl;

import com.alibaba.fastjson.JSONObject;
import com.malk.Util.UtilHttp;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.dingtalk.DDClient_New;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

@Slf4j
@Service
public class YDClient_DD implements com.malk.service.aliwork.YDClient_DD {

    @Autowired
    private YDConf ydConf;

    @Autowired
    private DDClient_New client_new;

    // 初始化请求参数
    Map _initBodyParam(YDParam ydParam) {
        if (StringUtils.isBlank(ydParam.getUserId())) {
            ydParam.setUserId(YDConf.PUB_ACCOUNT);
        }
        // 配置宜搭请求参数: appType, systemToken
        if (StringUtils.isBlank(ydParam.getAppType()) || ydParam.getAppType().equals(ydConf.getAppType())) {
            ydParam.setAppType(ydConf.getAppType());
            ydParam.setSystemToken(ydConf.getSystemToken());
        }
        return JSONObject.parseObject(JSONObject.toJSONString(ydParam), Map.class);
    }

    /**
     * 操作数据
     */
    @Override
    public Object operateData(YDParam ydParam, YDConf.TYPE_OPERATION type) {
        DDR_New ddr_new = null;
        String url = "https://api.dingtalk.com/v1.0/yida/forms/instances";
        switch (type) {
            case create:
                ddr_new = (DDR_New) UtilHttp.doPost(url, client_new.initTokenHeader(), null, _initBodyParam(ydParam), DDR_New.class);
                break;
            case update:
                ddr_new = (DDR_New) UtilHttp.doPut(url, client_new.initTokenHeader(), null, _initBodyParam(ydParam), DDR_New.class);
                break;
            case delete:
                ddr_new = (DDR_New) UtilHttp.doDelete(url + "/" + ydParam.getFormInstId(), client_new.initTokenHeader(), DDR_New.class);
                break;
            default:
                break;
        }
        return ddr_new.getData();
    }

    /**
     * 查询数据
     */
    @Override
    public DDR_New queryData(YDParam ydParam, YDConf.TYPE_QUERY type) {
        String host = "https://api.dingtalk.com/v1.0/yida/";
        String rsp = null;
        switch (type) {
            case retrieve_list_all:
                rsp = UtilHttp.doPost(host + "forms/instances/advances/queryAll", client_new.initTokenHeader(), null, _initBodyParam(ydParam));
                break;
            case retrieve_list:
                rsp = UtilHttp.doPost(host + "forms/instances/advances/query", client_new.initTokenHeader(), null, _initBodyParam(ydParam));
                break;
        }
        return DDR_New.formatResponse(rsp);
    }

    /**
     * 临时免登地址
     */
    @Override
    public String convertTemporaryUrl(String url, int timeout) {
        Map param = new HashMap();
        param.put("systemToken", ydConf.getSystemToken());
        param.put("userId", YDConf.PUB_ACCOUNT);
        param.put("fileUrl", url);          // URL在param上时, 需要编码 [UtilHttp已经做了编码] - URLEncoder.encode(url, "UTF-8")
        param.put("timeout", timeout);      // 默认1分钟, 最大24小时 [毫秒]
        String rsp = UtilHttp.doGet("https://api.dingtalk.com/v1.0/yida/apps/temporaryUrls/" + ydConf.getAppType(), client_new.initTokenHeader(), param);
        return String.valueOf(DDR_New.formatResponse(rsp).getResult());
    }

    @Override
    public String convertTemporaryUrl(String url) {
        return convertTemporaryUrl(url, 60000);
    }
}
