package com.malk.service.dingtalk;

import com.alibaba.fastjson.JSONObject;
import com.malk.server.dingtalk.DDApproveDTO;

import java.io.File;
import java.util.List;
import java.util.Map;

public interface DDClient {

    /**
     * 获取部门列表
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/obtain-the-department-list-v2
     */

    /**
     * 获取访问授权
     */
    String getAccessToken();

    /**
     * 获取指定用户的所有父部门列表
     */
    Map listParentByUser(String userId);

    /**
     * 获取部门详情
     */
    Map departmentDetail(Number deptId);

    /**
     * 查询用户详情
     */
    Map getUserInfo(String userId);

    /**
     * 发起审批实例
     */
    String createProcessInstance(DDApproveDTO approveDTO);

    /**
     * 获取审批实例详情
     */
    Map queryProcessInstanceId(String processInstanceId);

    /**
     * 撤回审批实例 NOTE: 终审通过无法撤销，若是假勤管理单据才能通知撤销
     *
     * @param processInstanceId 审批实例ID
     * @param isSystem          是否通过系统操作
     * @param remark            终止说明
     * @param operatingUserId   操作人的userid, 当isSystem为false时，该参数必传
     * @apiNote https://open.dingtalk.com/document/orgapp-server/terminate-a-workflow-by-using-an-instance-id
     */
    boolean terminateRunningApprove(String processInstanceId, boolean isSystem, String remark, String operatingUserId);

    /**
     * 通知审批撤销
     */
    boolean cancelRunningApprove(String userid, String approve_id);

    /**
     * 添加审批评论
     *
     * @param processInstanceId 审批实例ID
     * @param file              文件
     * @param text              评论的内容
     * @param commentUserid     评论人的userid
     */
    boolean commentApproveInstance(String processInstanceId, File file, String text, String commentUserid);

    /**
     * 获取推送失败列表: 推送失败钉钉记录, 获取后记录会被清空
     */
    List<Map> queryFailedList();

    /**
     * 发送工作通知
     *
     * @param agentId    微应用的AgentID
     * @param useridList 接收者的userid列表，最大用户列表长度100 [任一有值]
     * @param deptIdList 接收者的部门id列表，最大列表长度20。接收者是部门ID时，包括子部门下的所有用户  [任一有值]
     * @param toAllUser  是否发送给企业全部用户  [任一有值]
     * @param message    消息内容 [多种形式]
     */
    void sendCorpConversationMessage(String agentId, List<String> useridList, List<String> deptIdList, boolean toAllUser, JSONObject message);

    /**
     * 查询假期规则列表
     *
     * @param op_userid       当前企业内拥有OA审批应用权限的管理员的userId
     * @param vacation_source 假期来源。取值："all"：获取的是所有假期类型; ""：获取的是调用添加假期规则接口新建的假期
     */
    List<Map> queryVacationTypeList(String op_userid, boolean vacation_source);

    /**
     * 查询假期余额 [分页是根据游标控制]
     *
     * @param op_userid  当前企业内拥有OA审批应用权限的管理员的userId
     * @param leave_code 假期类型唯一标识 [查询假期规则列表]
     * @param userids    待查询的员工ID列表, 半角逗号相隔
     * @param offset     分页偏移，从0开始的非负整数
     * @param size       分页偏移，最大50
     */
    List<Map> queryVacationQuota(String op_userid, String leave_code, String userids, int offset, int size);
}

