package com.malk.service.dingtalk.impl;

import cn.hutool.core.util.ObjectUtil;
import com.malk.Util.UtilHttp;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.dingtalk.DDClient;
import com.malk.service.dingtalk.DDClient_New;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 新版本, 不依赖架包
 */
@Slf4j
@Service
public class DDClientImpl_New implements DDClient_New {

    @Autowired
    private DDClient ddClient;

    /**
     * token授权参数: 新版本
     */
    @Override
    public Map initTokenHeader() {
        Map param = new HashMap();
        param.put("x-acs-dingtalk-access-token", ddClient.getAccessToken());
        return param;
    }

    /**
     * 创建钉钉待办任务新版SDK
     *
     * @apiNote https://open.dingtalk.com/document/isvapp/add-dingtalk-to-do-task
     */
    @Override
    public DDR_New createTBTask(String createUserId, String subject, String description, long dueTime, List<String> executorIds, List<String> participantIds, Map detailUrl, boolean isOnlyShowExecutor, int priority, Map notifyConfigs) {
        String unionId = String.valueOf(ddClient.getUserInfo(createUserId).get("unionid"));
        Map param = new HashMap();
        param.put("operatorId", unionId);
        Map body = new HashMap();
        body.put("subject", subject);
        if (StringUtils.isNotBlank(description)) body.put("description", description);
        if (dueTime > 0) body.put("dueTime", dueTime);
        if (ObjectUtil.isNotNull(executorIds)) {
            List<String> executorUnionIds = new ArrayList<>();
            for (String userId : executorIds) {
                executorUnionIds.add(String.valueOf(ddClient.getUserInfo(userId).get("unionid")));
            }
            body.put("executorIds", executorUnionIds);
        }
        if (ObjectUtil.isNotNull(participantIds)) {
            List<String> participantUnionIds = new ArrayList<>();
            for (String userId : participantIds) {
                participantUnionIds.add(String.valueOf(ddClient.getUserInfo(userId).get("unionid")));
            }
            body.put("participantIds", participantUnionIds);
        }
        if (ObjectUtil.isNotNull(detailUrl)) body.put("detailUrl", detailUrl);
        body.put("isOnlyShowExecutor", isOnlyShowExecutor);
        body.put("priority", priority);
        if (ObjectUtil.isNotNull(detailUrl)) body.put("notifyConfigs", notifyConfigs);
        String rsp = UtilHttp.doPost("https://api.dingtalk.com/v1.0/todo/users/" + unionId + "/tasks", initTokenHeader(), param, body);
        return DDR_New.formatResponse(rsp);
    }
}