package com.malk.service.feishu;

import com.alibaba.fastjson.JSONArray;
import com.malk.server.feishu.FSApproveDTO;

import javax.validation.constraints.NotNull;
import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

public interface FSClient {

    /**
     * code2session 免登[小程序]
     */
    Map codeToSession(String authCode);

    /**
     * code2session 免登[网页]
     */
    Map codeToAuthorization(String authCode);

    /**
     * 获取单个用户信息
     */
    Map queryUserById(String userId);

    /**
     * 查询班表信息: 用户是否排班 [日期格式为yyyyMMdd]
     * -
     * BUG: 班表接口日期校验居然是按照31，如果本月不足31就会自动计算为下月的号数
     */
    List<Map> queryAttendanceForUserDailyShifts(String checkDateFrom, String checkDateTo, String... userIds);

    /**
     * 创建审批实例 NOTE: 若流程取值为页面联系人, 通过接口发起会报错, 若是此需求需要组装好审批流一并推送
     */
    String createProcessInstance(FSApproveDTO approveDTO);

    /**
     * 获取单个审批实例详情
     */
    Map queryProcessInstanceById(String processInstanceId);

    /**
     * 批量查询打卡流水记录 [日期格式为秒级时间戳]
     */
    List<Map> queryAttendanceFlows(String checkTimeFrom, String checkTimeTo, String... userIds);

    /**
     * 批量查询打卡流水记录 NOTE: 支持多userId查询, 组装返回格式为 user: List, 为定时任务降低查询频次
     */
    Map<String, List<Map>> queryAttendanceFlowsAssembleByUserId(String checkTimeFrom, String checkTimeTo, String... userIds);

    /**
     * 查看审批模板定义
     */
    Map getProcessTemplateDefine(String processCode);

    /**
     * 订阅审批模板
     */
    void subscribeProcessCode(String processCode);

    /**
     * 取消审批模板
     */
    void unsubscribeProcessCode(String processCode);

    /**
     * 审批结果写入: 将三果数据回写到飞书的考勤系统中
     *
     * @param userId  审批用户
     * @param date    审批作用时间
     * @param prop    目前支持加班-overtime_works、请假-leaves、出差-trips和外出-outs这四种审批结果的写入
     * @param content 对应的数据格式, 详见 FSParam 组装
     */
    Map createOverApproveState(String userId, String date, String prop, JSONArray content);

    /**
     * 通知审批状态更新 [因为审批结果是实例回调同意后再写入, 因此只需要撤销做同步即可]
     *
     * @param approvalId   审批实例 ID
     * @param approvalType 审批类型，leave：请假，out：外出，overtime：加班，trip：出差，remedy：补卡
     * @param status       审批状态，1：不通过，2：通过，4：撤销
     */
    Map updateProcessState(String approvalId, String approvalType, int status);

    /**
     * 获取部门信息列表
     */
    Map queryDepartmentList(String pageToken, String departmentId, boolean fetchChild, Consumer lambda);

    /**
     * 获取部门直属用户列表
     */
    Map queryUserListByDepartmentId(String pageToken, String departmentId, Consumer lambda);

    /**
     * 审批单据: 上传文件 [飞书不支持直接传地址] ppExt 1. 存储本地后拿到文件对象上传飞书, 2. 飞书为后台即可, 前端虽然可调用但会有白名单限制
     *
     * @param type    文件类型（image 或 attachment）
     * @param name    文件名（需包含文件扩展名，如“文件.doc”)
     * @param content 文件
     * @apiNote https://open.feishu.cn/document/ukTMukTMukTM/uUDOyUjL1gjM14SN4ITN
     */
    Map uploadFileForApprove(String name, String type, File content);

    /**
     * 查询统计数据 [日期格式为yyyyMMdd]
     *
     * @param statsType        统计类型  daily：日度统计, month：月度统计
     * @param startDate        开始时间
     * @param endDate          结束时间（时间间隔不超过 40 天）
     * @param userIds          查询的用户 ID 列表（用户数量不超过 20）: NOTE, 实测最大值为200
     * @param needHistory      是否需要历史数据
     * @param currentGroupOnly 是否需要历史数据
     */
    List<Map> queryUserAttendanceStatisticsData(@NotNull String statsType, @NotNull int startDate, @NotNull int endDate,
                                                @NotNull List<String> userIds, boolean needHistory, boolean currentGroupOnly);

    /**
     * 查询统计表头
     */
    Map<String, ?> queryUserAttendanceStatisticsHeader(@NotNull String statsType, @NotNull int startDate, @NotNull int endDate);
}
