package com.malk.service.teambition;

import java.util.List;
import java.util.Map;

public interface TBClient {

    /**
     * 查询企业项目模板
     *
     * @param ptIds 项目模版ID集合，逗号组合
     * @param q     模糊查询名字
     */
    List<Map> templateSearch(String ptIds, String q);

    /**
     * 搜索企业成员: 条件为空返回也为空
     *
     * @param query 用户名、用户拼音、用户名简拼、邮箱、手机号码 [实测 name, email, phone 有效; pinyin, py 无效, 且这两个字段在个人信息页面不透出]
     * @apiNote https://open.teambition.com/help/docs/5eb5431141b191001bcda95c
     */
    List<Map> userSearch(String query);

    /**
     * 更新企业成员
     *
     * @param operatorId 操作人 ID
     * @param members    格式: { userId, role, profile: { name, phone, email, ...} }, 成员角色，-1-外部成员 0-成员 1-管理员 2-拥有者
     * @apiNote https://open.teambition.com/docs/apis/6321c6ce912d20d3b5a48a5b
     */
    Map updateUser(String operatorId, List<Map> members);

    /**
     * 创建空白项目
     */
    Map projectCreate(String name, String operatorId);

    /**
     * 通过模板创建项目
     */
    Map projectCreateWithTemplate(String name, String templateId, String operatorId);


    /**
     * 查询项目
     *
     * @param projectIds 项目ID集合，逗号分隔
     * @param name       项目名字(模糊匹配)
     * @param sourceId   原始项目ID
     */
    List<Map> projectDetail(String projectIds, String name, String sourceId);

    /**
     * 更新项目 [增量]
     *
     * @param data String name, description, logo, visibility, startDate, endDate [visibility 项目可见性，project | organization]
     */
    Map projectUpdate(String projectId, Map<String, String> data, String operatorId);

    /**
     * 创建项目成员
     *
     * @param projectId  项目ID, 路径参数
     * @param userIds    项目成员userId列表
     * @param operatorId 操作人 ID
     * @apiNote https://open.teambition.com/docs/apis/6321c6d0912d20d3b5a498c0
     */
    List<Map> createProjectMember(List<String> userIds, String projectId, String operatorId);

    /**
     * 获取项目角色列表
     *
     * @return { result:  { level } }   项目角色等级: 0=成员, 1=管理员, 2=拥有者
     * @apiNote https://open.teambition.com/docs/apis/6321c6d1912d20d3b5a49b8f
     */
    List<Map> queryProjectRoles(String projectId);

    /**
     * 修改项目成员的角色
     *
     * @param roleIds 角色ID列表 [通过 获取项目角色列表 获取]
     * @return { result:  { role } }   项目角色: 0=成员, 1=管理员, 2=拥有者
     */
    List<Map> updateProjectMember(List<String> userIds, List<String> roleIds, String projectId);

    /**
     * 查询任务详情
     *
     * @param taskId       任务ID集合,使用逗号分隔,和parentTaskId冲突(选其一)
     * @param shortIds     任务短ID集合,使用逗号分隔
     * @param parentTaskId 父任务ID,和taskIds冲突(选其一)
     * @return { priority } 默认排序：“较低”：-10，“普通”：0，“紧急”：1，“非常紧急”：2; 后台调整：调整后按照显示顺序排序，若是新增就会3，4往下。按照 -10，0，1，2，3，4
     * @apiNote https://open.teambition.com/docs/apis/6321c6d2912d20d3b5a4a7b8
     */
    List<Map> queryTaskDetail(String taskId, String shortIds, String parentTaskId);

    /**
     * 更新任务截止时间
     *
     * @throws 204 若是传入dueDate与相同, 则会直接返回204报错, 若原来是年月日, 修改为同一天添加时分秒, 也属于不同时间
     */
    Map updateTeakDueDate(String taskId, String dueDate, String operatorId);

}